#ifndef __KNIGHT_H
#define __KNIGHT_H

#define KNIGHT_WIDTH 32
#define KNIGHT_HEIGHT 32
#define KGHOST_WIDTH 32
#define KGHOST_HEIGHT 32
#define INITIAL_COLOR "yellow"
#define VISITED_COLOR "blue"
#define CONSIDER_COLOR "green"
#define REACH_COLOR    "orange"
#define FLASH_COLOR    "red"
#define KNIGHT_COLOR   "black"
#define KGHOST_COLOR   "black"
#define LINE_COLOR     "red"

struct ChessPos
{
    int row, col;

};

class KnightTour
{
    private:
	int nvisited;
	int visited[8][8];
	int order[8][8];
	ChessPos curr_pos;

        int FindNThreats( ChessPos pos );
	int NextMoves( ChessPos pos, ChessPos next[8] );
	ChessPos GetNextMove( ChessPos curr );

    public:
	KnightTour() { nvisited = 0; curr_pos.row = curr_pos.col = 0; };
	void InitTour( ChessPos start );
	int  MakeNextMove( void );

	ChessPos GetCurrPos( void ) { return curr_pos; };
	int  GetNVisited( void )    { return nvisited; };
};

class Algo : public View
{
    private:
	Bitmap *knight;  /* Knight */
	Bitmap *kghost;  /* Knight's ghost */
	ChessPos k_pos;  /* Knight's current pos */
	Loc *locs[8][8]; /* Chess board's square's centers */
	Rectangle *squares[8][8]; /* Chess board's squares */
	Text *strs[8][8]; /* Labels used to display threat #s */
    public:
	Algo() { k_pos.row = k_pos.col = 0; };
	int Init( int, int );
	int Ready( void );
	int Visit( int, int, int, int );
	int Consider( int, int, int, int );
	int Reach( int, int, int, int );
	int DoneConsider( int, int, int, int, int );
	int DoneReach( int, int, int, int );
	int DoneAllConsider( int, int );
	int NextMove( int, int );
};

class Path : public View
{
    private:
	Bitmap *knight;  /* Knight */
	ChessPos k_pos;  /* Knight's current pos */
	Loc *locs[8][8]; /* Chess board's square's centers */
	Rectangle *squares[8][8]; /* Chess board's squares */
    public:
	Path() { k_pos.row = k_pos.col = 0; };
	int Init( int, int );
	int Ready( void );
	int Visit( int, int, int, int );
	int NextMove( int, int );
};

class MyAnimator : public Animator
{
    private:
	Algo algo;
	Path path;
    public:
	int Controller();
	MyAnimator()
	{
	    RegisterView( &algo );
	    RegisterView( &path );
	};
};

#endif

