/**********************************************************************

Greg Newton
CS 7390
February 10, 1996

header file which defines a specialized version of a minimum heap class

**********************************************************************/

#include <time.h>
//#include <sys/time.h>
#include "RandomVariable.h"

/**********************************************************************/

class HeapNode;
class MinHeap;

/**********************************************************************/

#define MAX_HEIGHT 5
#define MAX_NODES 31
#define INFINITY INT_MAX

/**********************************************************************/

class Graph
{
public:
  Graph(int nodes);
  Graph(Graph& g);

  void setNode(int i, char l, float xVal, float yVal);
  void addEdge(int f, int t, int w);
  void removeEdge(int f, int t);
  int getNNodes();
  char getLabel(int i);
  int edgeExists(int f, int t);
  int getWeight(int f, int t);
  int getEdges(int f, int* t, int* w);
  double getX(int i);
  double getY(int i);
  void print();

private:
  int nNodes;
  double* x;
  double* y;
  char* labels;
  int weights[MAX_NODES][MAX_NODES];
};

/**********************************************************************/

class HeapNode 
{
public:
  HeapNode();
  HeapNode(int i, int k);

  void setKey(int k);
  void setGraphIndex(int i);
  void setParent(int p);
  int getKey();
  int getGraphIndex();
  int getParent();

private:
  int nodeKey;
  int graphIndex;
  int nodeParent;
};
  

/**********************************************************************/

class MinHeap
{
public:
  MinHeap();
  
  void buildHeap(int n, HeapNode** h);
  void insert(HeapNode* h);
  void print();
  HeapNode* extractMin();
  HeapNode* min();
  void decreaseKey(HeapNode* h, int newKey);
  int empty();
  int getGraphIndex(int i); // returns the graph index of the node in heap position i
  int getHeapIndex(int i); // return the index of the heap node that has graph index i
  int index(HeapNode* h);
  int getParent(int i);
  void updateKeys(int graphIndex, Graph* g);
  void drawMiniature();

private:
  int nNodes;
  HeapNode* nodes[MAX_NODES];

  int findIndex(HeapNode* h);
  int left(int i);
  int right(int i);
  int parent(int i);
  void exchange(int i, int j);
  void heapify(int i);
  void clearHeap();
};

/**********************************************************************/


/**********************************************************************/

const int MAX = 64;
const double radius = 0.02;
const double speed = 0.02;
const double horizontal_spacing = 0.06;
const double vertical_spacing = 0.05;

#define REGULAR_COLOR 0
#define START_NODE_COLOR 1
#define WRONG_COLOR 2
#define RIGHT_COLOR 3

/**********************************************************************/

class myView : public View
{
public:
  int init();
  void addGraphNode(int i, double x, double y, char* label);
  void addGraphEdge(int i, int j, int weight);
  void showGhosts(int i);
  void ghostifyEdges();
  void hideGhosts(int i);
  void showInstruction(char* i);
  void removeInstruction();
  void migrate(int i, int j);
  void connect(int i, int j);
  void disconnect(int i, int j);
  int highlight(int i, int j, int okay);
  void swapNodes(int i, int j);
  void showComparison(int f, int t);
  void changeWeight(int i, int w);
  void goBack(int i);
  void moveToTop(int i);
  int pickStartNode();
  void addToMST(int f, int t);
  void animate();

private:
  int ng;

  // set locations for the graph nodes
  Loc* graphLocations[MAX];
  // set locations for the heap nodes
  Loc* heapLocations[MAX];

  int color[MAX];
  int start;

  // set that holds the parts of a graph node
  Set* graphNodes[MAX];
  // parts of the graph node
  Circle* graphCircles[MAX];
  Text* graphLabels[MAX];
  Text* graphWeights[MAX];

  // parts of the "ghost" graph nodes
  Circle* ghostGraphCircles[MAX];
  Text* ghostGraphLabels[MAX];

  // graph edges and labels
  Line* graphEdges[MAX][MAX];
  Text* graphEdgeLabels[MAX][MAX];

  // heap edges (an edge "belongs" to the child)
  Line* heapEdges[MAX];

  // instruction
  Text* instruction;
};

/**********************************************************************/

class historyView : public View
{
public:
  int init();
  void addHeap();
  void addHeapNode(int n, char* label, int weight);
  void animate();
private:
  int nHeaps;
  int xVal;
  int yVal;
  Loc* heapLocations[MAX];
  Circle* heapCircles[MAX][MAX];
  
};

/**********************************************************************/

class MyAnimator : public Animator 
{
public:
  int Controller();
  int pickStartNode();
private:
  myView v;
  historyView h;
};

/**********************************************************************/

