/***************************************************************/
/*							                                   */
/*	       		ConsoleEditProc.cpp	                           */
/*							                                   */
/***************************************************************/
/*       Copyright 1993, 1997 Georgia Institute of Technology 
                     -- John T. Stasko, Roman S. Khramets      */

#include <windows.h>
#include "Init.h"
#include "console_control.h"
#include "print.h"

extern WNDPROC EditWindowProc;
extern HWND hConsoleEditWnd;
extern HWND hConsole;
extern void CheckXEvents();

extern APP_CALLBACK AppQuitCB;

static int consoleEditState;
static char buffer[400];
static char *toScan;
static DWORD dwStartChar;

#define BUFSIZE   (sizeof(buffer)/sizeof(char))


LRESULT CALLBACK ConsoleEditProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam) {
	int nVirtKey;
	DWORD lKeyData;
	DWORD dwEndChar;
	DWORD dwLines;
	DWORD dwLineCharPos;
	HLOCAL hlText;
	char *text;
	DWORD textLength;
	DWORD diff;
	char ch;
	int strLen;
	
    switch (iMsg) {
		case WM_CREATE:
			consoleEditState = SUSPENDED;
			break;

		case WM_SETCURSOR:
			SetCursor(LoadCursor(NULL, IDC_ARROW));
			return 0;

		case WM_MOUSEMOVE:
		case WM_LBUTTONDBLCLK:
		case WM_LBUTTONDOWN:
		case WM_LBUTTONUP:
		case WM_MBUTTONDBLCLK:
		case WM_MBUTTONDOWN:
		case WM_MBUTTONUP:
		case WM_RBUTTONDBLCLK:
		case WM_RBUTTONDOWN:
		case WM_RBUTTONUP:
		case WM_MOUSEACTIVATE:
			return 0;

		case WM_KEYDOWN:
		case WM_KEYUP:
			switch( consoleEditState ) {
				case SUSPENDED:
					return 0;

				case OUTPUT:
					return 0;

				case INPUT:
					return 0;
			}
			break;

		case WM_CHAR:
			switch( consoleEditState ) {
				case SUSPENDED:
					return 0;

				case OUTPUT:
					break;

				case INPUT:
					ch = (char)wParam;
					switch( ch ) {
						case 10:
						case 13:
							dwLines = SendMessage(hConsoleEditWnd,
								EM_GETLINECOUNT, 0, 0);
							dwLineCharPos = SendMessage(hConsoleEditWnd,
								EM_LINEINDEX, (WPARAM)(dwLines-1), 0);

							// clear the buffer
							strLen = (int)SendMessage(hConsoleEditWnd,
								EM_LINELENGTH, (WPARAM)dwLineCharPos, (LPARAM)0);

							// first word of the buffer contains the max
							// number of chars that can be copied
							((WORD *)buffer)[0] = BUFSIZE - 1;
							SendMessage(hConsoleEditWnd,
								EM_GETLINE, (WPARAM)(dwLines-1), (LPARAM)buffer);
							buffer[BUFSIZE - 1 < strLen ? BUFSIZE - 1 : strLen] = '\0';
							toScan = &buffer[dwStartChar - dwLineCharPos];

							editSuspend();
							break;

						case '\b':
							dwEndChar = SendMessage(hwnd, WM_GETTEXTLENGTH, 0, 0);
							if( dwEndChar == dwStartChar ) {
								return 0;
							}
							break;
					}
					break;
			}
			break;

		case WM_KILLFOCUS:
			if( consoleEditState == INPUT ) {
				SetFocus(hConsole);
				return 0;
			}
			break;
	}

    return CallWindowProc(EditWindowProc, hwnd, iMsg, wParam, lParam);
}


void editSuspend() {
	consoleEditState = SUSPENDED;
}


void editOutput() {
	consoleEditState = OUTPUT;
}


char *editInput() {
    MSG  msg;

	CreateConsole();
	consoleEditState = INPUT;

	dwStartChar = SendMessage(hConsoleEditWnd,
		WM_GETTEXTLENGTH, 0, 0);

	SetFocus(hConsole);
	while(consoleEditState == INPUT) {
	    if( PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) ) {
	        TranslateMessage(&msg);
		    DispatchMessage(&msg);
		}
	}

	return toScan;
}


int getEditState() {
	return consoleEditState;
}


void setEditState(int aState) {
	consoleEditState = aState;
}
