/***************************************************************/
/*							                                   */
/*	       		ConsoleProc.cpp			                       */
/*							                                   */
/***************************************************************/
/*       Copyright 1997 Georgia Institute of Technology 
                     -- Roman S. Khramets                      */

#include <windows.h>
#include "console_control.h"
#include "Init.h"

extern HWND hConsole;
extern APP_CALLBACK AppQuitCB;
HWND hConsoleEditWnd;

LRESULT CALLBACK ConsoleProc (HWND hwnd, UINT iMsg, WPARAM wParam, LPARAM lParam) {
	 static RECT rcClient;
	 static HBRUSH hBkgBrush;
	 static LOGBRUSH lbBkgBrushInfo;
	 HDC hdcEdit;
     HWND hwndEdit;

     switch (iMsg) {
          case WM_CREATE :
			   hBkgBrush = (HBRUSH)GetStockObject(BLACK_BRUSH);
			   GetObject(hBkgBrush, sizeof(lbBkgBrushInfo), (LPVOID)&lbBkgBrushInfo);
			   GetClientRect(hwnd, &rcClient);
			   hConsoleEditWnd = CreateWindowEx(0,
				   "ConsoleEdit",
				   "",
				   WS_VISIBLE | WS_CHILD | ES_MULTILINE |
				   ES_AUTOHSCROLL | ES_AUTOVSCROLL |
				   ES_LEFT | ES_NOHIDESEL | ES_WANTRETURN |
				   WS_HSCROLL | WS_VSCROLL,
				   rcClient.left,
				   rcClient.top,
				   rcClient.right,
				   rcClient.bottom,
				   hwnd,
				   (HMENU)NULL,
				   ((LPCREATESTRUCT)lParam)->hInstance,
				   (LPVOID)lParam);
               return 0 ;

          case WM_SIZE :
               rcClient.left   = 0 ;
               rcClient.top    = 0 ;
               rcClient.right  = LOWORD (lParam) ;
               rcClient.bottom = HIWORD (lParam) ;

			   MoveWindow(hConsoleEditWnd,
				   rcClient.left,
				   rcClient.top,
				   rcClient.right,
				   rcClient.bottom,
				   TRUE);
               return 0 ;

		  case WM_COMMAND: {
			   int editState;
			   DWORD dwTextLength;
			   DWORD dwLineCount;
			   DWORD dwLineCharIndex;
			   WORD wNotifyCode = HIWORD(wParam); // notification code
		 	   WORD wID = LOWORD(wParam);         // item, control, or accelerator identifier
			   HWND hwndCtl = (HWND)lParam;       // handle of control

			   // oops, buffer overflow
			   if( wNotifyCode == EN_MAXTEXT ) {
				   // save the current ConsoleEdit's state
				   editState = getEditState();
				   // suspend the ConsoleEdit
				   editSuspend();
				   // get the total number of lines
				   dwLineCount = SendMessage(hwndCtl, EM_GETLINECOUNT,
					   (WPARAM)0, (LPARAM)0);
				   // get the character index of the line in the
				   // middle
				   dwLineCharIndex = SendMessage(hwndCtl, EM_LINEINDEX,
					   (WPARAM)(dwLineCount/2), (LPARAM)0);
				   // select all the text up to the middle line
				   SendMessage(hwndCtl, EM_SETSEL,
					   (WPARAM)0, (LPARAM)(dwLineCharIndex));
				   // replace the selection with an empty line
				   SendMessage(hwndCtl, EM_REPLACESEL,
					   (WPARAM)FALSE, (LPARAM)"");
				   // get the new text length
				   dwTextLength = SendMessage(hwndCtl,
					   WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0);
				   // move the current position to the end
				   // of the text
				   SendMessage(hwndCtl,
					   EM_SETSEL, (WPARAM)dwTextLength, (LPARAM)dwTextLength);
				   // go back into the saved state
				   setEditState(editState);
				   return 0;
			   } }
		       break;

		  case WM_SETFOCUS :
			   SetFocus(hConsoleEditWnd);
			   return 0 ;

		  case WM_CTLCOLOREDIT :
               hdcEdit = (HDC) wParam;   // handle of display context 
			   SetTextColor(hdcEdit, RGB(255, 255, 255));
			   SetBkColor(hdcEdit, lbBkgBrushInfo.lbColor);
			   return (LRESULT)hBkgBrush;

		  case WM_CLOSE:
			   AppQuitCB(NULL);
			   break;

		  case WM_DESTROY:
			   hConsole = NULL;
			   hConsoleEditWnd = NULL;
			   break;
	 }

     return DefWindowProc (hwnd, iMsg, wParam, lParam) ;
}
