/***************************************************************/
/*							                                   */
/*	       		print.cpp			                           */
/*							                                   */
/***************************************************************/
/*       Copyright 1997 Georgia Institute of Technology 
                     -- Roman S. Khramets                      */

#include <windows.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <io.h>
#include <fcntl.h>
#include <crtdbg.h>

#define	NO_REDEFS
#include "print.h"
#include "console_control.h"


extern "C" {
	int __cdecl _input(FILE *stream,
		const unsigned char *format,
		va_list arglist);
}


HWND hConsole;
extern HINSTANCE hApp;
extern HWND hConsoleEditWnd;

extern void cleanup();
extern void BeginModalState(HWND hWnd);


void CreateConsole(int aX, int aY, int aWidth, int aHeight) {
	static int x, y, width = 150, height = 400;

	if( hConsole == NULL ) {
		if( aWidth != 0 && aHeight != 0 ) {
			width = aWidth;
			height = aHeight;
		}

		x = aX;
		y = aY;

		hConsole = CreateWindowEx(0, "Console", "Console",
			WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU |
			WS_THICKFRAME | WS_MAXIMIZEBOX,
			x,
			y,
			width,
			height,
			NULL, NULL, hApp, NULL);
		ShowWindow(hConsole, SW_SHOW);
		UpdateWindow(hConsole);
	}
}


int __cdecl vsscanf(const char *string, const char *format, va_list arglist) {
	FILE str;
	register FILE *infile = &str;
	register int retval;

	_ASSERTE( string != NULL );
	_ASSERTE( format != NULL );

	infile->_flag = _IOREAD | _IOSTRG | _IOMYBUF;
	infile->_ptr = infile->_base = (char *)string;
	infile->_cnt = strlen(string);

	retval = _input(infile, (const unsigned char *)format, arglist);

	return retval;
}


int __win32_printf(const char *fmt, ...) {
    int rv;
    va_list arg;
	char buffer[1000];
	char curChar;
	int  curCharIndex;

	CreateConsole();

	editOutput();
    va_start(arg, fmt);

	DWORD dwInsertionPos;
	dwInsertionPos = SendMessage(hConsoleEditWnd,
		WM_GETTEXTLENGTH, (WPARAM)0, (LPARAM)0);
	SendMessage(hConsoleEditWnd,
		EM_SETSEL, (WPARAM)dwInsertionPos, (LPARAM)dwInsertionPos);

	rv = vsprintf(buffer, fmt, arg);
	curCharIndex = 0;
	while( (curChar = buffer[curCharIndex++]) ) {
		SendMessage(hConsoleEditWnd,
			WM_CHAR, (WPARAM)curChar, (LPARAM)1);
	}

    va_end(arg);
	editSuspend();

    return rv;
}


int __win32_scanf(const char *fmt, ...) {
    int rv;
    va_list arg;
	char *buffer;

	CreateConsole();

	buffer = editInput();
    va_start(arg, fmt);
	rv = vsscanf(buffer, fmt, arg);
    va_end(arg);

    return rv;
}


void __win32_exit(int rv) {
	cleanup();
    exit(rv);
}
